@@ -12,12 +12,11 @@
 int
 main(int argc, char **argv)
 {
-	struct hoedown_buffer *ib, *ob;
+	hoedown_buffer *ib, *ob;
 	FILE *in = stdin;
 
-	struct hoedown_callbacks callbacks;
-	struct hoedown_html_renderopt options;
-	struct hoedown_markdown *markdown;
+	hoedown_renderer *renderer;
+	hoedown_markdown *markdown;
 
 	/* opening the file if given from the command line */
 	if (argc > 1) {
@@ -41,11 +40,13 @@ main(int argc, char **argv)
 	/* performing markdown parsing */
 	ob = hoedown_buffer_new(OUTPUT_UNIT);
 
-	hoedown_html_renderer(&callbacks, &options, 0, 0);
-	markdown = hoedown_markdown_new(0, 16, &callbacks, &options);
+	renderer = hoedown_html_renderer_new(0, 0);
+	markdown = hoedown_markdown_new(0, 16, renderer);
 
 	hoedown_markdown_render(ob, ib->data, ib->size, markdown);
+
 	hoedown_markdown_free(markdown);
+	hoedown_html_renderer_free(renderer);
 
 	/* writing the result to stdout */
 	(void)fwrite(ob->data, 1, ob->size, stdout);
