@@ -9,6 +9,22 @@
 
 #define USE_XHTML(opt) (opt->flags & HOEDOWN_HTML_USE_XHTML)
 
+struct rndr_state {
+	struct {
+		int header_count;
+		int current_level;
+		int level_offset;
+		int nesting_level;
+	} toc_data;
+
+	unsigned int flags;
+
+	/* extra callbacks */
+	void (*link_attributes)(hoedown_buffer *ob, const hoedown_buffer *url, void *self);
+};
+
+typedef struct rndr_state rndr_state;
+
 int
 hoedown_html_is_tag(const uint8_t *tag_data, size_t tag_size, const char *tagname)
 {
@@ -42,12 +58,12 @@ hoedown_html_is_tag(const uint8_t *tag_data, size_t tag_size, const char *tagnam
 	return HOEDOWN_HTML_TAG_NONE;
 }
 
-static inline void escape_html(struct hoedown_buffer *ob, const uint8_t *source, size_t length)
+static inline void escape_html(hoedown_buffer *ob, const uint8_t *source, size_t length)
 {
 	hoedown_escape_html(ob, source, length, 0);
 }
 
-static inline void escape_href(struct hoedown_buffer *ob, const uint8_t *source, size_t length)
+static inline void escape_href(hoedown_buffer *ob, const uint8_t *source, size_t length)
 {
 	hoedown_escape_href(ob, source, length);
 }
@@ -56,29 +72,29 @@ static inline void escape_href(struct hoedown_buffer *ob, const uint8_t *source,
  * GENERIC RENDERER *
  ********************/
 static int
-rndr_autolink(struct hoedown_buffer *ob, const struct hoedown_buffer *link, enum hoedown_autolink type, void *opaque)
+rndr_autolink(hoedown_buffer *ob, const hoedown_buffer *link, enum hoedown_autolink type, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
 	if (!link || !link->size)
 		return 0;
 
-	if ((options->flags & HOEDOWN_HTML_SAFELINK) != 0 &&
+	if ((state->flags & HOEDOWN_HTML_SAFELINK) != 0 &&
 		!hoedown_autolink_is_safe(link->data, link->size) &&
 		type != HOEDOWN_AUTOLINK_EMAIL)
 		return 0;
 
-	BUFPUTSL(ob, "<a href=\"");
+	HOEDOWN_BUFPUTSL(ob, "<a href=\"");
 	if (type == HOEDOWN_AUTOLINK_EMAIL)
-		BUFPUTSL(ob, "mailto:");
+		HOEDOWN_BUFPUTSL(ob, "mailto:");
 	escape_href(ob, link->data, link->size);
 
-	if (options->link_attributes) {
+	if (state->link_attributes) {
 		hoedown_buffer_putc(ob, '\"');
-		options->link_attributes(ob, link, opaque);
+		state->link_attributes(ob, link, opaque);
 		hoedown_buffer_putc(ob, '>');
 	} else {
-		BUFPUTSL(ob, "\">");
+		HOEDOWN_BUFPUTSL(ob, "\">");
 	}
 
 	/*
@@ -92,25 +108,25 @@ rndr_autolink(struct hoedown_buffer *ob, const struct hoedown_buffer *link, enum
 		escape_html(ob, link->data, link->size);
 	}
 
-	BUFPUTSL(ob, "</a>");
+	HOEDOWN_BUFPUTSL(ob, "</a>");
 
 	return 1;
 }
 
 static void
-rndr_blockcode(struct hoedown_buffer *ob, const struct hoedown_buffer *text, const struct hoedown_buffer *lang, void *opaque)
+rndr_blockcode(hoedown_buffer *ob, const hoedown_buffer *text, const hoedown_buffer *lang, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
 
 	if (lang && lang->size) {
 		size_t i, cls = 0;
-		if (options->flags & HOEDOWN_HTML_PRETTIFY) {
-			BUFPUTSL(ob, "<pre><code class=\"prettyprint");
+		if (state->flags & HOEDOWN_HTML_PRETTIFY) {
+			HOEDOWN_BUFPUTSL(ob, "<pre><code class=\"prettyprint");
 			cls++;
 		} else {
-			BUFPUTSL(ob, "<pre><code class=\"");
+			HOEDOWN_BUFPUTSL(ob, "<pre><code class=\"");
 		}
 
 		for (i = 0; i < lang->size; ++i, ++cls) {
@@ -130,133 +146,133 @@ rndr_blockcode(struct hoedown_buffer *ob, const struct hoedown_buffer *text, con
 			}
 		}
 
-		BUFPUTSL(ob, "\">");
-	} else if (options->flags & HOEDOWN_HTML_PRETTIFY) {
-		BUFPUTSL(ob, "<pre><code class=\"prettyprint\">");
+		HOEDOWN_BUFPUTSL(ob, "\">");
+	} else if (state->flags & HOEDOWN_HTML_PRETTIFY) {
+		HOEDOWN_BUFPUTSL(ob, "<pre><code class=\"prettyprint\">");
 	} else {
-		BUFPUTSL(ob, "<pre><code>");
+		HOEDOWN_BUFPUTSL(ob, "<pre><code>");
 	}
 
 	if (text)
 		escape_html(ob, text->data, text->size);
 
-	BUFPUTSL(ob, "</code></pre>\n");
+	HOEDOWN_BUFPUTSL(ob, "</code></pre>\n");
 }
 
 static void
-rndr_blockquote(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_blockquote(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
-	BUFPUTSL(ob, "<blockquote>\n");
+	HOEDOWN_BUFPUTSL(ob, "<blockquote>\n");
 	if (text) hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</blockquote>\n");
+	HOEDOWN_BUFPUTSL(ob, "</blockquote>\n");
 }
 
 static int
-rndr_codespan(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_codespan(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
-	if (options->flags & HOEDOWN_HTML_PRETTIFY)
-		BUFPUTSL(ob, "<code class=\"prettyprint\">");
+	rndr_state *state = opaque;
+	if (state->flags & HOEDOWN_HTML_PRETTIFY)
+		HOEDOWN_BUFPUTSL(ob, "<code class=\"prettyprint\">");
 	else
-		BUFPUTSL(ob, "<code>");
+		HOEDOWN_BUFPUTSL(ob, "<code>");
 	if (text) escape_html(ob, text->data, text->size);
-	BUFPUTSL(ob, "</code>");
+	HOEDOWN_BUFPUTSL(ob, "</code>");
 	return 1;
 }
 
 static int
-rndr_strikethrough(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_strikethrough(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size)
 		return 0;
 
-	BUFPUTSL(ob, "<del>");
+	HOEDOWN_BUFPUTSL(ob, "<del>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</del>");
+	HOEDOWN_BUFPUTSL(ob, "</del>");
 	return 1;
 }
 
 static int
-rndr_double_emphasis(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_double_emphasis(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size)
 		return 0;
 
-	BUFPUTSL(ob, "<strong>");
+	HOEDOWN_BUFPUTSL(ob, "<strong>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</strong>");
+	HOEDOWN_BUFPUTSL(ob, "</strong>");
 
 	return 1;
 }
 
 static int
-rndr_emphasis(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_emphasis(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size) return 0;
-	BUFPUTSL(ob, "<em>");
+	HOEDOWN_BUFPUTSL(ob, "<em>");
 	if (text) hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</em>");
+	HOEDOWN_BUFPUTSL(ob, "</em>");
 	return 1;
 }
 
 static int
-rndr_underline(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_underline(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size)
 		return 0;
 
-	BUFPUTSL(ob, "<u>");
+	HOEDOWN_BUFPUTSL(ob, "<u>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</u>");
+	HOEDOWN_BUFPUTSL(ob, "</u>");
 
 	return 1;
 }
 
 static int
-rndr_highlight(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_highlight(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size)
 		return 0;
 
-	BUFPUTSL(ob, "<mark>");
+	HOEDOWN_BUFPUTSL(ob, "<mark>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</mark>");
+	HOEDOWN_BUFPUTSL(ob, "</mark>");
 
 	return 1;
 }
 
 static int
-rndr_quote(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_quote(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size)
 		return 0;
 
-	BUFPUTSL(ob, "<q>");
+	HOEDOWN_BUFPUTSL(ob, "<q>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</q>");
+	HOEDOWN_BUFPUTSL(ob, "</q>");
 
 	return 1;
 }
 
 static int
-rndr_linebreak(struct hoedown_buffer *ob, void *opaque)
+rndr_linebreak(hoedown_buffer *ob, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
-	hoedown_buffer_puts(ob, USE_XHTML(options) ? "<br/>\n" : "<br>\n");
+	rndr_state *state = opaque;
+	hoedown_buffer_puts(ob, USE_XHTML(state) ? "<br/>\n" : "<br>\n");
 	return 1;
 }
 
 static void
-rndr_header(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int level, void *opaque)
+rndr_header(hoedown_buffer *ob, const hoedown_buffer *text, int level, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
 	if (ob->size)
 		hoedown_buffer_putc(ob, '\n');
 
-	if ((options->flags & HOEDOWN_HTML_TOC) && (level <= options->toc_data.nesting_level))
-		hoedown_buffer_printf(ob, "<h%d id=\"toc_%d\">", level, options->toc_data.header_count++);
+	if ((state->flags & HOEDOWN_HTML_TOC) && (level <= state->toc_data.nesting_level))
+		hoedown_buffer_printf(ob, "<h%d id=\"toc_%d\">", level, state->toc_data.header_count++);
 	else
 		hoedown_buffer_printf(ob, "<h%d>", level);
 
@@ -265,38 +281,38 @@ rndr_header(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int le
 }
 
 static int
-rndr_link(struct hoedown_buffer *ob, const struct hoedown_buffer *link, const struct hoedown_buffer *title, const struct hoedown_buffer *content, void *opaque)
+rndr_link(hoedown_buffer *ob, const hoedown_buffer *link, const hoedown_buffer *title, const hoedown_buffer *content, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
-	if (link != NULL && (options->flags & HOEDOWN_HTML_SAFELINK) != 0 && !hoedown_autolink_is_safe(link->data, link->size))
+	if (link != NULL && (state->flags & HOEDOWN_HTML_SAFELINK) != 0 && !hoedown_autolink_is_safe(link->data, link->size))
 		return 0;
 
-	BUFPUTSL(ob, "<a href=\"");
+	HOEDOWN_BUFPUTSL(ob, "<a href=\"");
 
 	if (link && link->size)
 		escape_href(ob, link->data, link->size);
 
 	if (title && title->size) {
-		BUFPUTSL(ob, "\" title=\"");
+		HOEDOWN_BUFPUTSL(ob, "\" title=\"");
 		escape_html(ob, title->data, title->size);
 	}
 
-	if (options->link_attributes) {
+	if (state->link_attributes) {
 		hoedown_buffer_putc(ob, '\"');
-		options->link_attributes(ob, link, opaque);
+		state->link_attributes(ob, link, opaque);
 		hoedown_buffer_putc(ob, '>');
 	} else {
-		BUFPUTSL(ob, "\">");
+		HOEDOWN_BUFPUTSL(ob, "\">");
 	}
 
 	if (content && content->size) hoedown_buffer_put(ob, content->data, content->size);
-	BUFPUTSL(ob, "</a>");
+	HOEDOWN_BUFPUTSL(ob, "</a>");
 	return 1;
 }
 
 static void
-rndr_list(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int flags, void *opaque)
+rndr_list(hoedown_buffer *ob, const hoedown_buffer *text, int flags, void *opaque)
 {
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
 	hoedown_buffer_put(ob, flags & HOEDOWN_LIST_ORDERED ? "<ol>\n" : "<ul>\n", 5);
@@ -305,9 +321,9 @@ rndr_list(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int flag
 }
 
 static void
-rndr_listitem(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int flags, void *opaque)
+rndr_listitem(hoedown_buffer *ob, const hoedown_buffer *text, int flags, void *opaque)
 {
-	BUFPUTSL(ob, "<li>");
+	HOEDOWN_BUFPUTSL(ob, "<li>");
 	if (text) {
 		size_t size = text->size;
 		while (size && text->data[size - 1] == '\n')
@@ -315,13 +331,13 @@ rndr_listitem(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int
 
 		hoedown_buffer_put(ob, text->data, size);
 	}
-	BUFPUTSL(ob, "</li>\n");
+	HOEDOWN_BUFPUTSL(ob, "</li>\n");
 }
 
 static void
-rndr_paragraph(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_paragraph(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 	size_t i = 0;
 
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
@@ -334,8 +350,8 @@ rndr_paragraph(struct hoedown_buffer *ob, const struct hoedown_buffer *text, voi
 	if (i == text->size)
 		return;
 
-	BUFPUTSL(ob, "<p>");
-	if (options->flags & HOEDOWN_HTML_HARD_WRAP) {
+	HOEDOWN_BUFPUTSL(ob, "<p>");
+	if (state->flags & HOEDOWN_HTML_HARD_WRAP) {
 		size_t org;
 		while (i < text->size) {
 			org = i;
@@ -358,11 +374,11 @@ rndr_paragraph(struct hoedown_buffer *ob, const struct hoedown_buffer *text, voi
 	} else {
 		hoedown_buffer_put(ob, &text->data[i], text->size - i);
 	}
-	BUFPUTSL(ob, "</p>\n");
+	HOEDOWN_BUFPUTSL(ob, "</p>\n");
 }
 
 static void
-rndr_raw_block(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_raw_block(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	size_t org, sz;
 	if (!text) return;
@@ -377,68 +393,68 @@ rndr_raw_block(struct hoedown_buffer *ob, const struct hoedown_buffer *text, voi
 }
 
 static int
-rndr_triple_emphasis(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_triple_emphasis(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size) return 0;
-	BUFPUTSL(ob, "<strong><em>");
+	HOEDOWN_BUFPUTSL(ob, "<strong><em>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</em></strong>");
+	HOEDOWN_BUFPUTSL(ob, "</em></strong>");
 	return 1;
 }
 
 static void
-rndr_hrule(struct hoedown_buffer *ob, void *opaque)
+rndr_hrule(hoedown_buffer *ob, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
-	hoedown_buffer_puts(ob, USE_XHTML(options) ? "<hr/>\n" : "<hr>\n");
+	hoedown_buffer_puts(ob, USE_XHTML(state) ? "<hr/>\n" : "<hr>\n");
 }
 
 static int
-rndr_image(struct hoedown_buffer *ob, const struct hoedown_buffer *link, const struct hoedown_buffer *title, const struct hoedown_buffer *alt, void *opaque)
+rndr_image(hoedown_buffer *ob, const hoedown_buffer *link, const hoedown_buffer *title, const hoedown_buffer *alt, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 	if (!link || !link->size) return 0;
 
-	BUFPUTSL(ob, "<img src=\"");
+	HOEDOWN_BUFPUTSL(ob, "<img src=\"");
 	escape_href(ob, link->data, link->size);
-	BUFPUTSL(ob, "\" alt=\"");
+	HOEDOWN_BUFPUTSL(ob, "\" alt=\"");
 
 	if (alt && alt->size)
 		escape_html(ob, alt->data, alt->size);
 
 	if (title && title->size) {
-		BUFPUTSL(ob, "\" title=\"");
+		HOEDOWN_BUFPUTSL(ob, "\" title=\"");
 		escape_html(ob, title->data, title->size); }
 
-	hoedown_buffer_puts(ob, USE_XHTML(options) ? "\"/>" : "\">");
+	hoedown_buffer_puts(ob, USE_XHTML(state) ? "\"/>" : "\">");
 	return 1;
 }
 
 static int
-rndr_raw_html(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_raw_html(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
 	/* HTML_ESCAPE overrides SKIP_HTML, SKIP_STYLE, SKIP_LINKS and SKIP_IMAGES
 	* It doens't see if there are any valid tags, just escape all of them. */
-	if((options->flags & HOEDOWN_HTML_ESCAPE) != 0) {
+	if((state->flags & HOEDOWN_HTML_ESCAPE) != 0) {
 		escape_html(ob, text->data, text->size);
 		return 1;
 	}
 
-	if ((options->flags & HOEDOWN_HTML_SKIP_HTML) != 0)
+	if ((state->flags & HOEDOWN_HTML_SKIP_HTML) != 0)
 		return 1;
 
-	if ((options->flags & HOEDOWN_HTML_SKIP_STYLE) != 0 &&
+	if ((state->flags & HOEDOWN_HTML_SKIP_STYLE) != 0 &&
 		hoedown_html_is_tag(text->data, text->size, "style"))
 		return 1;
 
-	if ((options->flags & HOEDOWN_HTML_SKIP_LINKS) != 0 &&
+	if ((state->flags & HOEDOWN_HTML_SKIP_LINKS) != 0 &&
 		hoedown_html_is_tag(text->data, text->size, "a"))
 		return 1;
 
-	if ((options->flags & HOEDOWN_HTML_SKIP_IMAGES) != 0 &&
+	if ((state->flags & HOEDOWN_HTML_SKIP_IMAGES) != 0 &&
 		hoedown_html_is_tag(text->data, text->size, "img"))
 		return 1;
 
@@ -447,98 +463,98 @@ rndr_raw_html(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void
 }
 
 static void
-rndr_table(struct hoedown_buffer *ob, const struct hoedown_buffer *header, const struct hoedown_buffer *body, void *opaque)
+rndr_table(hoedown_buffer *ob, const hoedown_buffer *header, const hoedown_buffer *body, void *opaque)
 {
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
-	BUFPUTSL(ob, "<table><thead>\n");
+	HOEDOWN_BUFPUTSL(ob, "<table><thead>\n");
 	if (header)
 		hoedown_buffer_put(ob, header->data, header->size);
-	BUFPUTSL(ob, "</thead><tbody>\n");
+	HOEDOWN_BUFPUTSL(ob, "</thead><tbody>\n");
 	if (body)
 		hoedown_buffer_put(ob, body->data, body->size);
-	BUFPUTSL(ob, "</tbody></table>\n");
+	HOEDOWN_BUFPUTSL(ob, "</tbody></table>\n");
 }
 
 static void
-rndr_tablerow(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_tablerow(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
-	BUFPUTSL(ob, "<tr>\n");
+	HOEDOWN_BUFPUTSL(ob, "<tr>\n");
 	if (text)
 		hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</tr>\n");
+	HOEDOWN_BUFPUTSL(ob, "</tr>\n");
 }
 
 static void
-rndr_tablecell(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int flags, void *opaque)
+rndr_tablecell(hoedown_buffer *ob, const hoedown_buffer *text, int flags, void *opaque)
 {
 	if (flags & HOEDOWN_TABLE_HEADER) {
-		BUFPUTSL(ob, "<th");
+		HOEDOWN_BUFPUTSL(ob, "<th");
 	} else {
-		BUFPUTSL(ob, "<td");
+		HOEDOWN_BUFPUTSL(ob, "<td");
 	}
 
 	switch (flags & HOEDOWN_TABLE_ALIGNMASK) {
 	case HOEDOWN_TABLE_ALIGN_CENTER:
-		BUFPUTSL(ob, " style=\"text-align: center\">");
+		HOEDOWN_BUFPUTSL(ob, " style=\"text-align: center\">");
 		break;
 
 	case HOEDOWN_TABLE_ALIGN_L:
-		BUFPUTSL(ob, " style=\"text-align: left\">");
+		HOEDOWN_BUFPUTSL(ob, " style=\"text-align: left\">");
 		break;
 
 	case HOEDOWN_TABLE_ALIGN_R:
-		BUFPUTSL(ob, " style=\"text-align: right\">");
+		HOEDOWN_BUFPUTSL(ob, " style=\"text-align: right\">");
 		break;
 
 	default:
-		BUFPUTSL(ob, ">");
+		HOEDOWN_BUFPUTSL(ob, ">");
 	}
 
 	if (text)
 		hoedown_buffer_put(ob, text->data, text->size);
 
 	if (flags & HOEDOWN_TABLE_HEADER) {
-		BUFPUTSL(ob, "</th>\n");
+		HOEDOWN_BUFPUTSL(ob, "</th>\n");
 	} else {
-		BUFPUTSL(ob, "</td>\n");
+		HOEDOWN_BUFPUTSL(ob, "</td>\n");
 	}
 }
 
 static int
-rndr_superscript(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_superscript(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (!text || !text->size) return 0;
-	BUFPUTSL(ob, "<sup>");
+	HOEDOWN_BUFPUTSL(ob, "<sup>");
 	hoedown_buffer_put(ob, text->data, text->size);
-	BUFPUTSL(ob, "</sup>");
+	HOEDOWN_BUFPUTSL(ob, "</sup>");
 	return 1;
 }
 
 static void
-rndr_normal_text(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_normal_text(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
 	if (text)
 		escape_html(ob, text->data, text->size);
 }
 
 static void
-rndr_footnotes(struct hoedown_buffer *ob, const struct hoedown_buffer *text, void *opaque)
+rndr_footnotes(hoedown_buffer *ob, const hoedown_buffer *text, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
 	if (ob->size) hoedown_buffer_putc(ob, '\n');
-	BUFPUTSL(ob, "<div class=\"footnotes\">\n");
-	hoedown_buffer_puts(ob, USE_XHTML(options) ? "<hr/>\n" : "<hr>\n");
-	BUFPUTSL(ob, "<ol>\n");
+	HOEDOWN_BUFPUTSL(ob, "<div class=\"footnotes\">\n");
+	hoedown_buffer_puts(ob, USE_XHTML(state) ? "<hr/>\n" : "<hr>\n");
+	HOEDOWN_BUFPUTSL(ob, "<ol>\n");
 	
 	if (text)
 		hoedown_buffer_put(ob, text->data, text->size);
 	
-	BUFPUTSL(ob, "\n</ol>\n</div>\n");
+	HOEDOWN_BUFPUTSL(ob, "\n</ol>\n</div>\n");
 }
 
 static void
-rndr_footnote_def(struct hoedown_buffer *ob, const struct hoedown_buffer *text, unsigned int num, void *opaque)
+rndr_footnote_def(hoedown_buffer *ob, const hoedown_buffer *text, unsigned int num, void *opaque)
 {
 	size_t i = 0;
 	int pfound = 0;
@@ -564,53 +580,53 @@ rndr_footnote_def(struct hoedown_buffer *ob, const struct hoedown_buffer *text,
 	} else if (text) {
 		hoedown_buffer_put(ob, text->data, text->size);
 	}
-	BUFPUTSL(ob, "</li>\n");
+	HOEDOWN_BUFPUTSL(ob, "</li>\n");
 }
 
 static int
-rndr_footnote_ref(struct hoedown_buffer *ob, unsigned int num, void *opaque)
+rndr_footnote_ref(hoedown_buffer *ob, unsigned int num, void *opaque)
 {
 	hoedown_buffer_printf(ob, "<sup id=\"fnref%d\"><a href=\"#fn%d\" rel=\"footnote\">%d</a></sup>", num, num, num);
 	return 1;
 }
 
 static void
-toc_header(struct hoedown_buffer *ob, const struct hoedown_buffer *text, int level, void *opaque)
+toc_header(hoedown_buffer *ob, const hoedown_buffer *text, int level, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
-	if (level <= options->toc_data.nesting_level) {
+	if (level <= state->toc_data.nesting_level) {
 		/* set the level offset if this is the first header
 		 * we're parsing for the document */
-		if (options->toc_data.current_level == 0)
-			options->toc_data.level_offset = level - 1;
+		if (state->toc_data.current_level == 0)
+			state->toc_data.level_offset = level - 1;
 
-		level -= options->toc_data.level_offset;
+		level -= state->toc_data.level_offset;
 
-		if (level > options->toc_data.current_level) {
-			while (level > options->toc_data.current_level) {
-				BUFPUTSL(ob, "<ul>\n<li>\n");
-				options->toc_data.current_level++;
+		if (level > state->toc_data.current_level) {
+			while (level > state->toc_data.current_level) {
+				HOEDOWN_BUFPUTSL(ob, "<ul>\n<li>\n");
+				state->toc_data.current_level++;
 			}
-		} else if (level < options->toc_data.current_level) {
-			BUFPUTSL(ob, "</li>\n");
-			while (level < options->toc_data.current_level) {
-				BUFPUTSL(ob, "</ul>\n</li>\n");
-				options->toc_data.current_level--;
+		} else if (level < state->toc_data.current_level) {
+			HOEDOWN_BUFPUTSL(ob, "</li>\n");
+			while (level < state->toc_data.current_level) {
+				HOEDOWN_BUFPUTSL(ob, "</ul>\n</li>\n");
+				state->toc_data.current_level--;
 			}
-			BUFPUTSL(ob,"<li>\n");
+			HOEDOWN_BUFPUTSL(ob,"<li>\n");
 		} else {
-			BUFPUTSL(ob,"</li>\n<li>\n");
+			HOEDOWN_BUFPUTSL(ob,"</li>\n<li>\n");
 		}
 
-		hoedown_buffer_printf(ob, "<a href=\"#toc_%d\">", options->toc_data.header_count++);
+		hoedown_buffer_printf(ob, "<a href=\"#toc_%d\">", state->toc_data.header_count++);
 		if (text) escape_html(ob, text->data, text->size);
-		BUFPUTSL(ob, "</a>\n");
+		HOEDOWN_BUFPUTSL(ob, "</a>\n");
 	}
 }
 
 static int
-toc_link(struct hoedown_buffer *ob, const struct hoedown_buffer *link, const struct hoedown_buffer *title, const struct hoedown_buffer *content, void *opaque)
+toc_link(hoedown_buffer *ob, const hoedown_buffer *link, const hoedown_buffer *title, const hoedown_buffer *content, void *opaque)
 {
 	if (content && content->size)
 		hoedown_buffer_put(ob, content->data, content->size);
@@ -618,20 +634,20 @@ toc_link(struct hoedown_buffer *ob, const struct hoedown_buffer *link, const str
 }
 
 static void
-toc_finalize(struct hoedown_buffer *ob, void *opaque)
+toc_finalize(hoedown_buffer *ob, void *opaque)
 {
-	struct hoedown_html_renderopt *options = opaque;
+	rndr_state *state = opaque;
 
-	while (options->toc_data.current_level > 0) {
-		BUFPUTSL(ob, "</li>\n</ul>\n");
-		options->toc_data.current_level--;
+	while (state->toc_data.current_level > 0) {
+		HOEDOWN_BUFPUTSL(ob, "</li>\n</ul>\n");
+		state->toc_data.current_level--;
 	}
 }
 
-void
-hoedown_html_toc_renderer(struct hoedown_callbacks *callbacks, struct hoedown_html_renderopt *options, int nesting_level)
+hoedown_renderer *
+hoedown_html_toc_renderer_new(int nesting_level)
 {
-	static const struct hoedown_callbacks cb_default = {
+	static const hoedown_renderer cb_default = {
 		NULL,
 		NULL,
 		NULL,
@@ -667,19 +683,42 @@ hoedown_html_toc_renderer(struct hoedown_callbacks *callbacks, struct hoedown_ht
 
 		NULL,
 		toc_finalize,
+		
+		NULL
 	};
 
-	memset(options, 0x0, sizeof(struct hoedown_html_renderopt));
-	options->flags = HOEDOWN_HTML_TOC;
-	options->toc_data.nesting_level = nesting_level;
+	rndr_state       *state;
+	hoedown_renderer *renderer;
 
-	memcpy(callbacks, &cb_default, sizeof(struct hoedown_callbacks));
+	/* Prepare the state pointer */
+	state = malloc(sizeof(rndr_state));
+	if (!state)
+		return NULL;
+
+	memset(state, 0x0, sizeof(rndr_state));
+
+	if (nesting_level > 0) {
+		state->flags |= HOEDOWN_HTML_TOC;
+		state->toc_data.nesting_level = nesting_level;
+	}
+
+	/* Prepare the renderer */
+	renderer = malloc(sizeof(hoedown_renderer));
+	if (!renderer) {
+		free(state);
+		return NULL;
+	}
+
+	memcpy(renderer, &cb_default, sizeof(hoedown_renderer));
+	
+	renderer->opaque = state;
+	return renderer;
 }
 
-void
-hoedown_html_renderer(struct hoedown_callbacks *callbacks, struct hoedown_html_renderopt *options, unsigned int render_flags, int toc_nesting_lvl)
+hoedown_renderer *
+hoedown_html_renderer_new(unsigned int render_flags, int nesting_level)
 {
-	static const struct hoedown_callbacks cb_default = {
+	static const hoedown_renderer cb_default = {
 		rndr_blockcode,
 		rndr_blockquote,
 		rndr_raw_block,
@@ -715,28 +754,54 @@ hoedown_html_renderer(struct hoedown_callbacks *callbacks, struct hoedown_html_r
 
 		NULL,
 		NULL,
+		
+		NULL
 	};
 
-	/* Prepare the options pointer */
-	memset(options, 0x0, sizeof(struct hoedown_html_renderopt));
-	options->flags = render_flags;
+	rndr_state       *state;
+	hoedown_renderer *renderer;
 
-	if (toc_nesting_lvl > 0) {
-		options->flags |= HOEDOWN_HTML_TOC;
-		options->toc_data.nesting_level = toc_nesting_lvl;
+	/* Prepare the state pointer */
+	state = malloc(sizeof(rndr_state));
+	if (!state)
+		return NULL;
+
+	memset(state, 0x0, sizeof(rndr_state));
+
+	state->flags = render_flags;
+
+	if (nesting_level > 0) {
+		state->flags |= HOEDOWN_HTML_TOC;
+		state->toc_data.nesting_level = nesting_level;
 	}
 
-	/* Prepare the callbacks */
-	memcpy(callbacks, &cb_default, sizeof(struct hoedown_callbacks));
+	/* Prepare the renderer */
+	renderer = malloc(sizeof(hoedown_renderer));
+	if (!renderer) {
+		free(state);
+		return NULL;
+	}
+
+	memcpy(renderer, &cb_default, sizeof(hoedown_renderer));
 
 	if (render_flags & HOEDOWN_HTML_SKIP_IMAGES)
-		callbacks->image = NULL;
+		renderer->image = NULL;
 
 	if (render_flags & HOEDOWN_HTML_SKIP_LINKS) {
-		callbacks->link = NULL;
-		callbacks->autolink = NULL;
+		renderer->link = NULL;
+		renderer->autolink = NULL;
 	}
 
 	if (render_flags & HOEDOWN_HTML_SKIP_HTML || render_flags & HOEDOWN_HTML_ESCAPE)
-		callbacks->blockhtml = NULL;
+		renderer->blockhtml = NULL;
+	
+	renderer->opaque = state;
+	return renderer;
+}
+
+void
+hoedown_html_renderer_free(hoedown_renderer *renderer)
+{
+	free(renderer->opaque);
+	free(renderer);
 }
