@@ -31,31 +31,20 @@ typedef enum {
 	HOEDOWN_HTML_TAG_CLOSE
 } hoedown_html_tag;
 
-struct hoedown_html_renderopt {
-	struct {
-		int header_count;
-		int current_level;
-		int level_offset;
-		int nesting_level;
-	} toc_data;
-
-	unsigned int flags;
-
-	/* extra callbacks */
-	void (*link_attributes)(struct hoedown_buffer *ob, const struct hoedown_buffer *url, void *self);
-};
-
 int
 hoedown_html_is_tag(const uint8_t *tag_data, size_t tag_size, const char *tagname);
 
-extern void
-hoedown_html_renderer(struct hoedown_callbacks *callbacks, struct hoedown_html_renderopt *options, unsigned int render_flags, int toc_nesting_lvl);
+extern hoedown_renderer *
+hoedown_html_renderer_new(unsigned int render_flags, int nesting_level);
+
+extern hoedown_renderer *
+hoedown_html_toc_renderer_new(int nesting_level);
 
 extern void
-hoedown_html_toc_renderer(struct hoedown_callbacks *callbacks, struct hoedown_html_renderopt *options, int nesting_level);
+hoedown_html_renderer_free(hoedown_renderer *renderer);
 
 extern void
-hoedown_html_smartypants(struct hoedown_buffer *ob, const uint8_t *text, size_t size);
+hoedown_html_smartypants(hoedown_buffer *ob, const uint8_t *text, size_t size);
 
 #ifdef __cplusplus
 }
